/**
 * Unsere Implementierung der Klasse Node, also eines Knotens. Das Besondere
 * hier ist, dass der Schlssel als Comparable abgespeichert wird. Das hat den
 * Vorteil, dass man alle mglichen Objekte miteinander vergleichen kann. Dies
 * brauchen wir fr die binre Suchbaumeigenschaft des Rot-Schwarz-Baumes. 
 */
public class Node
{
	/**
	 * Spezifiziert einen Knoten mit der Farbe Rot.
	 */
	public static final int RED = 1;
	
	/**
	 * Spezifiziert einen Knoten mit der Farbe Schwarz.
	 */
	public static final int BLACK = 2;
	
	private Node parent;
	private Node left;
	private Node right;
	private int color;
	private Comparable key;
	
	// Max-Wert fr Intervallbume
	private int max;

	/**
	 * Standard Konstruktor fr die Klasse Node. Alle Objekte innerhalb des
	 * Node-Objekts sind null.
	 */
	public Node()
	{
		parent = null;
		left = null;
		right = null;
		color = 0;
		key = null;
		
		max = 0;
	}
	
	/**
	 * Konstruktor fr die Klasse Node. Das Objekt Key erhlt eine Referenz,
	 * alle anderen internen Objekte sind null.
	 * @param key Schlssel des Node Objekts, der das Interface Comparable 
	 *            implementiert
	 */
	public Node(Comparable key)
	{
		parent = null;
		left = null;
		right = null;
		color = 0;
		this.key = key;
		
		max = 0;
	}
	
	/**
	 * Vollstndiger Konstruktor der Klasse Node. Jedem internen Objekt wird 
	 * eine Referenz zugeordnet.
	 * @param parent Vater Knoten dieses Knotens
	 * @param left Linker Knoten dieses Knotens
	 * @param right Rechter Knoten dieses Knotens
	 * @param color Farbe (rot oder schwarz) dieses Knotens
	 * @param key Schlssel des Node Objekts, der das Interface Comparable 
	 *            implementiert.
	 * @param max Max-Wert fr Intervallbume
	 */
	public Node(Node parent, Node left, Node right, int color, Comparable key, int max)
	{
		this.parent = parent;
		this.left = left;
		this.right = right;
		this.color = color;
		this.key = key;
		this.max = max;
	}
	
	/**
	 * Gibt die Farbe (rot oder schwarz) des Knotens zurck
	 * @return Die Farbe (rot oder schwarz) vom Typ int
	 */
	public int getColor()
	{
		return color;
	}
	
	/**
	 * Setzt die Farbe (rot oder schwarz) des Knotens
	 * @param color Die neue Farbe (rot oder schwarz)
	 */
	public void setColor(int color)
	{
		this.color = color;
	}
	
	/**
	 * Gibt den Schlssel des Knotens zurck, der das Interface Comparable 
	 * implementiert.
	 * @return Der Schlssel des Knotens
	 */
	public Comparable getKey()
	{
		return key;
	}
	
	/**
	 * Setzt den Schlssel des Knotens, der das Interface Comparable 
	 * implementiert.
	 * @param key Der neue Schlssel
	 */
	public void setKey(Comparable key)
	{
		this.key = key;
	}
	
	/**
	 * Gibt den linken Knoten dieses Knotens zurck.
	 * @return Der linke Kind-Knoten
	 */
	public Node getLeft()
	{
		return left;
	}
	
	/**
	 * Setzt den linken Knoten dieses Knotens.
	 * @param left Der neue linke Kind-Knoten
	 */
	public void setLeft(Node left)
	{
		this.left = left;
	}
	
	/**
	 * Gibt den Vater Knoten dieses Knotens zurck.
	 * @return Der Vater Knoten
	 */
	public Node getParent() 
	{
		return parent;
	}
	
	/**
	 * Setzt den Vater Knoten dieses Knotens.
	 * @param parent Der neue Vater Knoten
	 */
	public void setParent(Node parent) 
	{
		this.parent = parent;
	}
	
	/**
	 * Gibt den rechten Knoten dieses Knotens zurck.
	 * @return Der rechte Kind-Knoten
	 */
	public Node getRight() 
	{
		return right;
	}
	
	/**
	 * Setzt den rechten Knoten dieses Knotens.
	 * @param right Der neue rechte Kind-Knoten
	 */
	public void setRight(Node right) 
	{
		this.right = right;
	}
	
	/**
	 * Gibt den Max-Wert dieses Knotens zurck.
	 * @return Max-Wert
	 */
	public int getMax()	
	{
		return max;
	}
	
	/**
	 * Setzt den Max-Wert dieses Knotens auf den spezifizierten Wert.
	 * @param max Der neue Max-Wert
	 */
	public void setMax(int max)	
	{
		this.max = max;
	}
	
	/** 
	 * Gibt einen String zurck, welcher eine Reprsentation dieses Knotens
	 * enthlt. Als erstes erfolgt eine Stringreprsentation des Schlssels.
	 * Ist der Schlssel vom Typ Interval, wird auch der max Wert an den String
	 * angehngt.
	 * Bsp:
	 * 6 (ohne Interval)
	 * [3,9] 9 (mit Interval)
	 * @return eine Stringreprsentation dieses Knotens 
	 */
	public String toString() 
	{
		String tmp = key.toString();
    /*
		if (key instanceof Interval )
		{
			tmp = tmp + " " + max; 
		}*/
		return tmp;
	}
}