/**
 * Diese Klasse beschreibt einen grundlegenden RotSchwarz-Baum. In den Baum 
 * knnen Instanzen der Klasse "Node" gespeichert werden. Die Implementierung
 * benutzt die Sentinel-Technik, das heisst, alle leeren Objektreferenzen 
 * zeigen auf den Pseudo-Knoten "nil".
 * Ein Rot-Schwarz-Baum verhlt sich wie ein normaler binrer Suchbaum, jedoch
 * wird er aufgrund der Farben (Rot und Schwarz) der einzelnen Knoten 
 * ausbalanciert, da folgende Eigenschaften immer gelten mssen:
 * <pre>
 * 1. Die Wurzel ist schwarz
 * 2. Jedes Blatt(NIL) ist schwarz
 * 3. Die Kinder eines roten Knotens sind schwarz
 * 4. Alle Bltter haben dieselbe Schwarztiefe</pre>
 */
public class RedBlackTree
{
	// Der Wurzelknoten und der Sentinel
	private Node root;
	private Node nil;

	/**
	 * Standard Konstruktor fr die Klasse RedBlackTree. Dieser erzeugt einen
	 * leeren Baum.
	 */
	public RedBlackTree()
	{
		// NIL-Knoten vorinitialisieren
		nil = new Node();
		/*nil.setParent(nil);
		nil.setRight(nil);
		nil.setLeft(nil);*/
		nil.setColor(Node.BLACK);
		
		// Am Anfang ist der Baum leer: root == nil
		root = nil;
	}
	
	/**
	 * Sucht den Schlssel des bergebenen Knotens in dem Baum. Wenn der 
	 * Schlssel im Baum enthalten ist, wird der entsprechende Knoten zurck
	 * gegeben, ansonsten wird der NIL-Knoten zurchgegeben. 
	 * @param node Spezifiziert den zu suchenden Schlssel
	 * @return Den gefundenen Knoten, falls der Schlssel im Baum ist, sonst
	 *         NIL
	 */
	public Node search(Node node)
	{
		Node x = root;
		
		// Setzen des gesuchten Schlssels in den NIL-Knoten
		// Dadurch wird der Schlssel auf jeden Fall gefunden
		nil.setKey(node.getKey());
		
		while (x.getKey().compareTo(node.getKey()) != 0)
		{
			if (x.getKey().compareTo(node.getKey()) > 0)
			{
				x = x.getLeft();
			}
			else
			{
				x = x.getRight();
			}
		}
		
		// Zurcksetzen des NIL-Knotens
		nil.setKey(new Integer(0));
    
		return x;
	}
	
	/**
	 * Fgt den bergebenen Knoten in den Baum ein, und zwar so, dass eine 
	 * Inorder-Traversierung die Schlssel in aufsteigender Reihenfolge 
	 * sortiert durchluft. (Binre Suchbaumeigenschaft) 
	 * @param node Der Knoten, der in den Baum eingefgt werden soll
	 */
	public void insert(Node node)
	{
		Node x = root;
		Node y = nil;
		
		// Suchen der Einfgestelle
		while (x != nil)
		{
			y = x;
			if (node.getKey().compareTo(x.getKey()) < 0)
			{
				x = x.getLeft();
			}
			else
			{
				x = x.getRight();
			}
		}
		
		// Einfgen des Knotens
		node.setParent(y);
		if (y == nil)
		{
			root = node;
		}
		else
		{
			if (node.getKey().compareTo(y.getKey()) < 0)
			{
				y.setLeft(node);
			}
			else
			{
				y.setRight(node);
			}
		}
    
		node.setLeft(nil);
		node.setRight(nil);
		node.setColor(Node.RED);
		insertFixup(node);
		
		/*nil.setParent(nil);
		nil.setRight(nil);
		nil.setLeft(nil);
		nil.setColor(Node.BLACK);*/
	}
	
	/**
	 * Lscht den bergebenen Knoten aus dem Baum und balanciert den Baum 
	 * danach wieder aus. 
	 * @param node Der zu lschende Knoten
	 * @return Der Knoten der gelscht wurde, oder NIL, falls ein solcher 
	 *         Knoten nicht existierte
	 */
	public Node delete(Node node)
	{
		Node x = null;
		Node y = null;
		
		// Ist der Knoten berhaupt im Baum vorhanden?
		node = search(node);
		if (node == nil)
		{
			return nil;
		}
		
		// wenn ja, dann lsche den Knoten
		if (node.getLeft() == nil || node.getRight() == nil)
		{
			y = node;
		}
		else
		{
			y = successor(node);
		}
		
		if (y.getLeft() != nil)
		{
			x = y.getLeft();
		}
		else
		{
			x = y.getRight();
		}
		
		x.setParent(y.getParent());
		if (y.getParent() == nil)
		{
			root = x;
		}
		else
		{
			if (y == y.getParent().getLeft())
			{
				y.getParent().setLeft(x);
			}
			else
			{
				y.getParent().setRight(x);
			}
		}
		
		if (y != node)
		{
			node.setKey(y.getKey());
			// Copy satellite data
		}
		
		// Ausbalancieren des Baums.
		if (y.getColor() == Node.BLACK)
		{
			deleteFixup(x);
		}
		
		/*nil.setParent(nil);
		nil.setRight(nil);
		nil.setLeft(nil);
		nil.setColor(Node.BLACK);*/
		
		return y;
	}
	
	/**
	 * Gibt den Nachfolger des bergebenen Knotens zurck oder den NIL-Knoten,
	 * falls ein solcher Knoten nicht existiert. Nachfolger bedeutet, dass der 
	 * Schlssel des Nachfolgerknotens auf den Schlssel des bergebenen 
	 * Knotens folgt. Als Vergleichskriterium wird die compareTo-Methode der 
	 * Schlssel benutzt. 
	 * @param x Der Knoten, fr den der Nachfolger gesucht wird.
	 * @return Der Nachfolgerknoten oder NIL, falls ein solcher Knoten nicht 
	 *         existiert
	 */
	public Node successor(Node x)
	{
		Node y = nil;
		
		// Wenn es einen rechten Teilbaum gibt, dann muss sich der Nachfolger
		// in diesem befinden, und zwar ganz links in diesem Teilbaum
    
		if (x.getRight() != null&&x.getRight() != nil)
		{
			y = x.getRight();
      //System.out.println("Hallo"+x.getLeft());
			while (y.getLeft() != nil)
			{
				y = y.getLeft();
        
			}
		}
		else
		{
			// Ansonsten befindet sich der Nachfolger in einem Parentknoten
			y = x.getParent();
			while (y!=null&&y != nil && y.getKey().compareTo(x.getKey()) <= 0)
			{
      
				y = y.getParent();
			}
		}
    
		return y;
	}
	

    
    
	/**
	 * Gibt den Vorgnger des bergebenen Knotens zurck oder NIL, falls ein
	 * solcher Knoten nicht existiert. Vorgnger bedeutet, dass der Schlssel
	 * des Vorgngerknotens vor dem Schlssel des bergebenen Knotens liegt.
	 * Als Vergleichskriterium wird die compareTo-Methode der Schlssel benutzt. 
	 * @param x Der Knoten, fr den der Vorgnger gesucht wird.
	 * @return Der Vorgngerknoten oder NIL, falls ein solcher Knoten nicht 
	 *         existiert
	 */
	public Node predecessor(Node x)
	{
		Node y = nil;
		
		// Wenn es einen linken Teilbaum gibt, dann muss sich der Vorgnger
		// in diesem befinden, und zwar ganz rechts in diesem Teilbaum
		if (x.getLeft() != null&&x.getLeft() != nil)
		{
			y = x.getLeft();
			while (y.getRight() != nil)
			{
				y = y.getRight();
			}
		}
		else
		{
			// Ansonsten befindet sich der Vorgnger in einem Parentknoten
			y = x.getParent();
			while (y!=null&&y != nil && y.getKey().compareTo(x.getKey()) >= 0)
			{
				y = y.getParent();
        
			}
		}
    
		return y;
	}
	
	/**
	 * Gibt der Wurzelknoten des Rot-Schwarzbaumes zurck.
	 * @return der Wurzelknoten
	 */
	public Node getRoot()
	{
		return root;
	}
	
	/**
	 * Gibt der NIL-Knoten des Rot-Schwarzbaumes zurck.
	 * @return der NIL-Knoten
	 */
	public Node getNil()
	{
		return nil;
	}
	
	/**
	 * Bei einer Linksrotation wird der rechte Knoten von node dessen parent
	 * und der linke Teilbaum vom rechten Knoten der neue rechte Teilbaum von
	 * node. 
	 * Die Methode ist protected, damit ableitende Klasse diese berschreiben 
	 * knnen, um eigene Funktionalitten hinzuzufgen.
	 * @param node Der Knoten, an dem die Linksrotation vorgenommen wird
	 */
	protected void leftRotate(Node node)
	{
		Node y = node.getRight();
		node.setRight(y.getLeft());
		y.getLeft().setParent(node);
		y.setParent(node.getParent());
		if (node.getParent() == nil)
		{
			root = y;
		}
		else
		{
			if (node == node.getParent().getLeft())
			{
				node.getParent().setLeft(y);
			}
			else
			{
				node.getParent().setRight(y);
			}
		}
		y.setLeft(node);
		node.setParent(y);	
	}
	
	/**
	 * Bei einer Rechtsrotation wird der linke Knoten von node dessen parent
	 * und der rechte Teilbaum vom linkeKnoten der neue linke Teilbaum von
	 * node. 
	 * Die Methode ist protected, damit ableitende Klasse diese berschreiben 
	 * knnen, um eigene Funktionalitten hinzuzufgen.
	 * @param node Der Knoten, an dem die Rechtsrotation vorgenommen wird
	 */
	protected void rightRotate(Node node)
	{
		Node x = node.getLeft();
		node.setLeft(x.getRight());
		if (x.getRight()!=null) x.getRight().setParent(node);
		x.setParent(node.getParent());
		if (node.getParent() == nil)
		{
			root = x;
		}
		else
		{
			if (node == node.getParent().getLeft())
			{
				node.getParent().setLeft(x);
			}
			else
			{
				node.getParent().setRight(x);
			}
		}
		x.setRight(node);
		node.setParent(x);
	}

	// Die InsertFixup-Methode balanciert den Baum nach einer Einfgeoperation
	// wieder aus.
	private void insertFixup(Node node)
	{
		Node y;
		while (node.getParent().getColor() == Node.RED)
		{
			if (node.getParent() == node.getParent().getParent().getLeft())
			{
				y = node.getParent().getParent().getRight();
				if (y.getColor() == Node.RED)
				{
					node.getParent().setColor(Node.BLACK);
					y.setColor(Node.BLACK);
					node.getParent().getParent().setColor(Node.RED);
					node = node.getParent().getParent();
				}
				else
				{
					if (node == node.getParent().getRight())
					{
						node = node.getParent();
 						leftRotate(node);
					}
					node.getParent().setColor(Node.BLACK);
					node.getParent().getParent().setColor(Node.RED);
					rightRotate(node.getParent().getParent());
				}
			}
			else
			{
				y = node.getParent().getParent().getLeft();
				if (y.getColor() == Node.RED)
				{
					node.getParent().setColor(Node.BLACK);
					y.setColor(Node.BLACK);
					node.getParent().getParent().setColor(Node.RED);
					node = node.getParent().getParent();
				}
				else
				{
					if (node == node.getParent().getLeft())
					{
						node = node.getParent();
 						rightRotate(node);
					}
					node.getParent().setColor(Node.BLACK);
					node.getParent().getParent().setColor(Node.RED);
					leftRotate(node.getParent().getParent());
				}
			}
		}
		
		root.setColor(Node.BLACK);
	}
	
	// Die DeleteFixup-Methode balanciert den Baum nach einer Lschenoperation 
	// wieder aus.
	private void deleteFixup(Node node)
	{
		Node w = null;
		while (node != root && node.getColor() == Node.BLACK)
		{
			if (node == node.getParent().getLeft())
			{
				w = node.getParent().getRight();
				if (w.getColor() == Node.RED)
				{
					w.setColor(Node.BLACK);
					node.getParent().setColor(Node.RED);
					leftRotate(node.getParent());
					w = node.getParent().getRight();
				}
				if (w.getLeft().getColor() == Node.BLACK && w.getRight().getColor() == Node.BLACK)
				{
					w.setColor(Node.RED);
					node = node.getParent();
				}
				else
				{
					if (w.getRight().getColor() == Node.BLACK)
					{
						w.getLeft().setColor(Node.BLACK);
						w.setColor(Node.RED);
						rightRotate(w);
						w = node.getParent().getRight();
					}
					
					w.setColor(node.getParent().getColor());
					node.getParent().setColor(Node.BLACK);
					if (w.getRight()!=null) w.getRight().setColor(Node.BLACK);
					if (node.getParent()!=null) leftRotate(node.getParent());
					node = root;
				}
			}
			else
			{
				w = node.getParent().getLeft();
				if (w.getColor() == Node.RED)
				{
					w.setColor(Node.BLACK);
					node.getParent().setColor(Node.RED);
					rightRotate(node.getParent());
					w = node.getParent().getLeft();
				}
				if (w.getRight().getColor() == Node.BLACK && w.getLeft().getColor() == Node.BLACK)
				{
					w.setColor(Node.RED);
					node = node.getParent();
				}
				else
				{
					if (w.getLeft().getColor() == Node.BLACK)
					{
						w.getRight().setColor(Node.BLACK);
						w.setColor(Node.RED);
						leftRotate(w);
						w = node.getParent().getLeft();
					}
					
					w.setColor(node.getParent().getColor());
					node.getParent().setColor(Node.BLACK);
					if (w.getLeft()!=null) w.getLeft().setColor(Node.BLACK);
					rightRotate(node.getParent());
					node = root;
				}
			}
		}
		
		node.setColor(Node.BLACK);
	}
}